@@ -8,13 +8,17 @@ import warnings
 import string
 import time
 import enum
-from typing import NamedTuple, Any, List, Optional
+import ipaddress
+from typing import NamedTuple, Any, List, Optional, Dict, Union, TYPE_CHECKING, Tuple
 from ast import literal_eval
 from decimal import Decimal, localcontext, InvalidOperation as InvalidDecimalOperation
 from itertools import repeat
 from orderly_set import StableSetEq as SetOrderedBase  # median: 1.0867 s for cache test, 5.63s for all tests
 from threading import Timer
 
+if TYPE_CHECKING:
+    from pytz.tzinfo import BaseTzInfo
+
 
 class np_type:
     pass
@@ -54,6 +58,7 @@ except ImportError:  # pragma: no cover. The case without Numpy is tested locall
     np_complex128 = np_type  # pragma: no cover.
     np_cdouble = np_type  # pragma: no cover.
     np_complexfloating = np_type  # pragma: no cover.
+    np_datetime64 = np_type  # pragma: no cover.
 else:
     np_array_factory = np.array
     np_ndarray = np.ndarray
@@ -76,6 +81,7 @@ else:
     np_complex128 = np.complex128
     np_cdouble = np.cdouble  # np.complex_ is an alias for np.cdouble and is being removed by NumPy 2.0
     np_complexfloating = np.complexfloating
+    np_datetime64 = np.datetime64
 
 numpy_numbers = (
     np_int8, np_int16, np_int32, np_int64, np_uint8,
@@ -89,6 +95,7 @@ numpy_complex_numbers = (
 
 numpy_dtypes = set(numpy_numbers)
 numpy_dtypes.add(np_bool_)  # type: ignore
+numpy_dtypes.add(np_datetime64)  # type: ignore
 
 numpy_dtype_str_to_type = {
     item.__name__: item for item in numpy_dtypes
@@ -180,10 +187,11 @@ unicode_type = str
 bytes_type = bytes
 only_complex_number = (complex,) + numpy_complex_numbers
 only_numbers = (int, float, complex, Decimal) + numpy_numbers
-datetimes = (datetime.datetime, datetime.date, datetime.timedelta, datetime.time)
+datetimes = (datetime.datetime, datetime.date, datetime.timedelta, datetime.time, np_datetime64)
+ipranges = (ipaddress.IPv4Interface, ipaddress.IPv6Interface, ipaddress.IPv4Network, ipaddress.IPv6Network)
 uuids = (uuid.UUID, )
-times = (datetime.datetime, datetime.time)
-numbers = only_numbers + datetimes
+times = (datetime.datetime, datetime.time,np_datetime64)
+numbers: Tuple = only_numbers + datetimes
 booleans = (bool, np_bool_)
 
 basic_types = strings + numbers + uuids + booleans + (type(None), )
@@ -605,11 +613,17 @@ def literal_eval_extended(item):
         raise
 
 
-def time_to_seconds(t):
+def time_to_seconds(t:datetime.time) -> int:
     return (t.hour * 60 + t.minute) * 60 + t.second
 
 
-def datetime_normalize(truncate_datetime, obj):
+def datetime_normalize(
+    truncate_datetime:Union[str, None],
+    obj:Union[datetime.datetime, datetime.time],
+    default_timezone: Union[
+        datetime.timezone, "BaseTzInfo"
+    ] = datetime.timezone.utc,
+) -> Any:
     if truncate_datetime:
         if truncate_datetime == 'second':
             obj = obj.replace(microsecond=0)
@@ -621,11 +635,11 @@ def datetime_normalize(truncate_datetime, obj):
             obj = obj.replace(hour=0, minute=0, second=0, microsecond=0)
     if isinstance(obj, datetime.datetime):
         if has_timezone(obj):
-            obj = obj.astimezone(datetime.timezone.utc)
+            obj = obj.astimezone(default_timezone)
         else:
-            obj = obj.replace(tzinfo=datetime.timezone.utc)
+            obj = obj.replace(tzinfo=default_timezone)
     elif isinstance(obj, datetime.time):
-        obj = time_to_seconds(obj)
+        return time_to_seconds(obj)
     return obj
 
 
@@ -643,7 +657,7 @@ def has_timezone(dt):
     return dt.tzinfo is not None and dt.tzinfo.utcoffset(dt) is not None
 
 
-def get_truncate_datetime(truncate_datetime):
+def get_truncate_datetime(truncate_datetime) -> Union[str, None]:
     """
     Validates truncate_datetime value
     """
@@ -722,13 +736,17 @@ def detailed__dict__(obj, ignore_private_variables=True, ignore_keys=frozenset()
                 ignore_private_variables and key.startswith('__') and not key.startswith(private_var_prefix)
             ):
                 del result[key]
+        if isinstance(obj, PydanticBaseModel):
+            getter = lambda x, y: getattr(type(x), y)
+        else:
+            getter = getattr
         for key in dir(obj):
             if key not in result and key not in ignore_keys and (
                     not ignore_private_variables or (
                         ignore_private_variables and not key.startswith('__') and not key.startswith(private_var_prefix)
                     )
             ):
-                value = getattr(obj, key)
+                value = getter(obj, key)
                 if not callable(value):
                     result[key] = value
     return result
@@ -785,6 +803,7 @@ class FlatDataAction(EnumBase):
     attribute_added = 'attribute_added'
     unordered_iterable_item_added = 'unordered_iterable_item_added'
     unordered_iterable_item_removed = 'unordered_iterable_item_removed'
+    initiated = "initiated"
 
 
 OPCODE_TAG_TO_FLAT_DATA_ACTION = {
@@ -797,7 +816,7 @@ OPCODE_TAG_TO_FLAT_DATA_ACTION = {
 FLAT_DATA_ACTION_TO_OPCODE_TAG = {v: i for i, v in OPCODE_TAG_TO_FLAT_DATA_ACTION.items()}
 
 
-UnkownValueCode = 'unknown___'
+UnkownValueCode: str = 'unknown___'
 
 
 class FlatDeltaRow(NamedTuple):
@@ -814,3 +833,12 @@ class FlatDeltaRow(NamedTuple):
     t2_to_index: Optional[int] = None
 
     __repr__ = __str__ = named_tuple_repr
+
+
+JSON = Union[Dict[str, str], List[str], List[int], Dict[str, "JSON"], List["JSON"], str, int, float, bool, None]
+
+
+class SummaryNodeType(EnumBase):
+    dict = 'dict'
+    list = 'list'
+    leaf = 'leaf'
